/*simpleflt.c by Oscar Pablo Di Liscia*/

#include "../sndlib-l/io.c"
#include "../sndlib-l/headers.c"
#include "../sndlib-l/sound.c"
#include "../my_funcs/io_funcs.h"
#include "simpleflt.h"

int main(int argc, char *argv[])
{
  SOUND_DATA *in_data=NULL,*out_data=NULL;
  mus_sample_t **obuf, **ibuf;
  int i;
  int maxargs;

  void process(SOUND_DATA *in,SOUND_DATA *out, mus_sample_t **ibuf,mus_sample_t **obuf, char * args[]);
  
  maxargs=BUTTER_ARGS;

  if(checkin(argc, maxargs+3, "Command line: simpleflt infile outfile args", "SIMPLEFLT")==-1) {
    getchar();
    return(-1);
  }

  mus_sound_initialize();
  
  if((in_data=open_input(argv[1],  "\nINPUT FILE:"))==NULL)
    {return(-1);}
  if((out_data=open_output(argv[2],"\nOUTPUT FILE:",in_data))==NULL)
    {return(-1);}
  
  /*RESERVAR MEMORIA*/
  ibuf=snd_memalloc(in_data, BUFFER_SIZE);
  obuf=snd_memalloc(out_data, BUFFER_SIZE);
  
  //fdata=(FDATA*)fltalloc(ONEZERO_SIZE, FIR);
  //set_one_zero(fdata, atof(argv[3]) , atof(argv[4]));
  //fdata=(FDATA*)fltalloc(ONEPOLE_SIZE, IIR);
  //set_one_pole(fdata, atof(argv[3]) , atof(argv[4]));
  //fdata =(FDATA*)alloc_comb(atoi(argv[3]));
  //set_comb(fdata,atoi(argv[3]),atof(argv[4]),atof(argv[5]));
  //fdata=(FDATA*)fltalloc(RESON_SIZE, IIR);
  //set_reson(atof(argv[3]), atof(argv[4]),fdata, in_data,atof(argv[5]));
  
  /*PROCESAR*/
  process(in_data, out_data, ibuf, obuf, argv);

  /*LIBERAR MEMORIA, CERRAR ARCHIVOS*/
  snd_dealloc(in_data, BUFFER_SIZE, ibuf);
  snd_dealloc(out_data, BUFFER_SIZE, obuf);
  close(in_data->fd);
  close(out_data->fd);
  free(in_data);
  free(out_data);
 
  return(0);
}

/***************************************************************/
void process(SOUND_DATA *in,SOUND_DATA *out, mus_sample_t **ibuf,mus_sample_t **obuf, char * args[] )
{
  int i,k,j;
  int size=BUFFER_SIZE;
  float insample, outsample;
  float maxgain=0.;
  float ant=0.;
  FDATA *fdata1, *fdata2, *fdata3;
  
  fdata1=(FDATA*)fltalloc(BUTTER_SIZE, IIR);
  set_butter(atof(args[3]), fdata1, in, atoi(args[4]),atof(args[5]));
  fdata2=(FDATA*)fltalloc(BUTTER_SIZE, IIR);
  set_butter(atof(args[3]), fdata2, in, atoi(args[4]),atof(args[5]));
  fdata3=(FDATA*)fltalloc(BUTTER_SIZE, IIR);
  set_butter(atof(args[3]), fdata3, in, atoi(args[4]),atof(args[5]));

  for (i=0; i < out->frames; i+=BUFFER_SIZE) {
      mus_sound_read (in->fd, 0,size-1,in->chans, ibuf);    
      for (k=0; k<in->chans; k++) {
      
	for (j=0; j<size; j++) {
	  if(i+j == out->frames) break; 
	  /*HACER LO QUE UNO QUIERA (O PUEDA)CON obuf AQUI*/
	  insample=MUS_SAMPLE_TO_FLOAT(ibuf[k][j]);
	  outsample=butter(butter(butter(insample, fdata1),fdata2),fdata3);
	  obuf[k][j]=MUS_FLOAT_TO_SAMPLE(outsample);

	  maxgain=(outsample > maxgain ? outsample : maxgain);
	}

      }  
      if(j != BUFFER_SIZE && j !=0) size=j-1;
	mus_sound_write(out->fd,0,size-1,out->chans,obuf);
  }
  mus_sound_close_input (in->fd); 
  mus_sound_close_output(out->fd, out->frames * mus_bytes_per_sample(out->format));
 
  fltdealloc(fdata1, IIR);
  fltdealloc(fdata2, IIR);
  fltdealloc(fdata3, IIR);

  fprintf(stdout,"\n Maxgain= %f",maxgain);
  fprintf(stdout,"\n\n %ld frames of %ld samples each one written", ((i-BUFFER_SIZE)+j),out->chans );
  return;
}
/***************************************************************/
