/*simpleflt.h by Oscar Pablo Di Liscia*/


#define ONEZERO_ARGS  2 /*a0 a1*/
#define ONEPOLE_ARGS  2 /*a0 b1*/
#define BUTTER_ARGS   3 /*frec type gain*/
#define COMB_ARGS     3 /*delay feedback gain*/
#define RESON_ARGS    3 /*FC BW gain*/

#define FIR       0
#define IIR       1

#define MAX_TAPS  30      

#define ONEZERO   1
#define ONEPOLE   2
#define RESON     3
#define BUTTER    4
#define COMB      5
#define ALPASS    6
#define DELAY     7
#define VDELAY    8

#define LP       0
#define HP       1

#define ONEZERO_SIZE   2
#define ONEPOLE_SIZE   2
#define RESON_SIZE     4
#define BUTTER_SIZE    3


typedef struct {
  int   L;
  float *ih;
  float *oh;
  float *a;
  float *b;
  int   bptr;
  float gain;
}FDATA;

typedef struct {
  int   L;
  int   cnt;
  float *buffer;
  int   ptr[MAX_TAPS];
}DDATA;

double PI = M_PI;
double PI2= 2. * M_PI;

//double PI=4*atan(1.);

/******************************************************************/
/******************************************************************/
FDATA  *fltalloc (int length, int type);
void   fltdealloc (FDATA *fdata, int type);
void   set_one_zero(FDATA *fdata, float a0, float a1);
void   set_one_pole(FDATA *fdata, float a0, float b0);
float  one_zero (float input, FDATA *fdata);
float  one_pole (float input, FDATA *fdata);
void   set_comb(FDATA *fdata, int delay, float gain, float feedback);
FDATA  *alloc_comb(int delay);
float  comb(float input, FDATA *fdata);
void   set_butter(float frec, FDATA *fdata, SOUND_DATA *sdata, int type, float gain);
float  butter(float in, FDATA *fdata);
void   set_reson(float frec, float bw, FDATA *fdata, SOUND_DATA *sdata, float gain);
float  reson(float input, FDATA *fdata);
void   w_delay(float input , DDATA *del);
DDATA  *set_delay(int L);
float  r_delay(DDATA *del, float delay, int tapcnt);
float  fdelay(float input, DDATA *del, float delay, int tapcnt);


/******************************************************************/
/******************************************************************/
FDATA  *fltalloc (int length, int type)
{
  FDATA  *ptr=NULL;
  int i;

  ptr= (FDATA *)malloc(sizeof(FDATA));
  if(!ptr) return(NULL);

  
  ptr->ih= (float*)malloc(sizeof(float)*length+1);
  ptr->a = (float*)malloc(sizeof(float)*length+1);

  if(type == IIR) {
    ptr->oh= (float*)malloc(sizeof(float)*length+1);
    ptr->b = (float*)malloc(sizeof(float)*length+1);
  }

  ptr->L=length;

  for(i=0; i<ptr->L; ++i) { 
    
    ptr->ih[i]=0. ;
    ptr->a[i] =0. ;
    
    if(type == IIR) {
      ptr->oh[i]=0.;
      ptr->b[i] =0.;
    }
  }
  
  return(ptr);
}
/******************************************************************/
void  fltdealloc (FDATA *fdata, int type)
{
   
  free(fdata->ih);
  free(fdata->a );

  if(type == IIR) {
    free(fdata->oh);
    free(fdata->b );
  }

  free(fdata);
   
  return;
}

/******************************************************************/
void set_one_zero(FDATA *fdata, float a0, float a1)
{
  fdata->a[0]=a0;
  fdata->a[1]=a1;
  return;
}
/******************************************************************/
void set_one_pole(FDATA *fdata, float a0, float b0)
{
  fdata->a[0]=a0;
  fdata->b[0]=b0;
  return;
}
/******************************************************************/
float one_zero (float input, FDATA *fdata)
{
  float output;

  output= input*fdata->a[0] + fdata->a[1]*fdata->ih[0];
  fdata->ih[0]=input*fdata->a[0];

  return(output);
}
/******************************************************************/
float one_pole (float input, FDATA *fdata)
{
  float output;

  output= fdata->a[0]*input +  fdata->b[0]*fdata->oh[0];
  fdata->oh[0]=output;

  return(output);
}
/******************************************************************/
FDATA *alloc_comb(int delay)
{
  FDATA  *ptr=NULL;
  ptr= (FDATA *)malloc(sizeof(FDATA));
  if(!ptr) return(NULL);

  ptr->ih= (float*)malloc(sizeof(float));
  ptr->a = (float*)malloc(sizeof(float));
  ptr->oh= (float*)malloc(sizeof(float)*delay);
  ptr->b = (float*)malloc(sizeof(float));

  return(ptr);
}
/******************************************************************/
void set_comb(FDATA *fdata, int delay, float gain, float feedback)
{
  fdata->L   =delay;
  fdata->a[0]=gain;
  fdata->b[0]=feedback;
  fdata->bptr=0;
  
  return;
}
/******************************************************************/
float comb(float input, FDATA *fdata)
{
  float output;

  output= fdata->a[0]*input +  fdata->b[0]*fdata->oh[fdata->bptr];
  fdata->oh[fdata->bptr]=output;

  if(++fdata->bptr == fdata->L)
    {fdata->bptr=0;}

  return(output);
}

/******************************************************************/
void set_butter(float frec, FDATA *fdata, SOUND_DATA *sdata, int type, float gain)
{
  float ce, cee;

   //setup coefficients according to frec (frec is the desired cut frequency in Hertz)
  switch(type)  { //lowpass or  hipass

  case LP: 
    ce          = 1. / tan( (double)((float)(PI/(double)sdata->srate)*frec));
    cee         = ce*ce;
    fdata->a[0] = 1. / (1. + (ce * sqrt(2.)) + cee);
    fdata->a[1] = fdata->a[0] * 2.;
    fdata->a[2]=  fdata->a[0];
    fdata->b[0]=  2. * (1. - cee) * fdata->a[0];
    fdata->b[1]=  ((1. - (ce * sqrt(2.))) + cee) * fdata->a[0];
    break;
    
  case HP: 
    ce     =	tan( (double)((float)(PI/(double)sdata->srate)*frec ));
    cee    =    ce*ce;
    fdata->a[0] = 1. / (1. + (ce * sqrt(2.)) + cee);
    fdata->a[1] = fdata->a[0] * -2.;
    fdata->a[2]=  fdata->a[0];
    fdata->b[0]=  2. * (cee - 1.) * fdata->a[0];
    fdata->b[1]=  ((1. - (ce * sqrt(2.))) + cee) * fdata->a[0];
    break;
  }

  fdata->gain=gain;

  return;	
}
/********************************************************************/
float butter(float in, FDATA *fdata)
{
float out;

 fdata->ih[0]   =in - fdata->ih[1]*fdata->b[0] - fdata->ih[2]*fdata->b[1];  
 out	        =fdata->ih[0]*fdata->a[0] +  fdata->ih[1]*fdata->a[1] + fdata->ih[2]*fdata->a[2];

 fdata->ih[2]	=fdata->ih[1];
 fdata->ih[1]	=fdata->ih[0];

return(out*fdata->gain);
}
/******************************************************************/
void set_reson(float frec, float bw, FDATA *fdata, SOUND_DATA *sdata, float gain)
{
  /*
    R = exp(-PI*bw/SR)
    G = 1-R
    b1= 2R * cos(2*PI*f/SR)
    b2= -R*R
    (aqui usamos a[0] para guardar G) y gain para guardar el ajuste general
    de ganancia
  */
  
  
  fdata->a[2]=(float)exp(-PI*bw/(double)sdata->srate);
  fdata->a[0]=1. - fdata->a[2];
  fdata->b[1]= 2 * fdata->a[2] * cos(2.*PI*frec/(double)sdata->srate);
  fdata->b[2]= - (fdata->a[2]*fdata->a[2]);
  fdata->gain=gain;

  return;
}
/******************************************************************/
float  reson(float input, FDATA *fdata)
{
  float output;
  /*y(n)= G*[x(n) - R*x(n-2)] + b1*y(n-1) + b2*y(n-2)*/
 
  output= fdata->a[0]*(input - fdata->a[2]*fdata->ih[2]) + 
          fdata->oh[1]*fdata->b[1] + fdata->oh[2]*fdata->b[2];
  
  fdata->oh[2]=fdata->oh[1];
  fdata->oh[1]=output;
  fdata->ih[2]=fdata->ih[1];
  fdata->ih[1]=input;

  return(output*fdata->gain);
}
/******************************************************************/
DDATA *set_delay(int L)
{

  DDATA *del;
  int i;
  
  del        = (DDATA*)malloc(sizeof(DDATA));
  del->buffer= (float*)malloc(sizeof(float)*L);
  del->L     = L;
  del->cnt   = 0;
  for(i=0; i<MAX_TAPS; ++i) {
    del->ptr[i]=0;
  }

  return(del);
}
/******************************************************************/
void  w_delay(float input , DDATA *del)
{
  int i;

  del->buffer[del->cnt]=input;

  if(--del->cnt < 0) {  
    del->cnt=del->L-1;
  }
  
  return;
}

/******************************************************************/
float  r_delay(DDATA *del, float delay, int tapcnt)
{
/*OJO: el delay esta en samples*/

  float output;
  int t1, t2;

  t1= del->ptr[tapcnt] + (int)delay;
  while(t1 >= del->L)  t1-=del->L;

  t2= t1+1;
  while(t2 >= del->L)  t2-=del->L;
  
  output=del->buffer[t1] + (delay-((int)delay))* (del->buffer[t1]*del->buffer[t2]); 
  
  if(--del->ptr[tapcnt] < 0)
    del->ptr[tapcnt]=del->L-1;

  return(output);

  return;
}
/******************************************************************/
float fdelay(float input, DDATA *del, float delay, int tapcnt)
{
  float output;

  if(delay==0.) return(input);

  if(++del->cnt >= del->L)
    del->cnt =0;

  output=del->buffer[del->cnt];
  del->buffer[del->cnt]=input;

  return(output);
}

/*para trabajar...*/
/**********************************************************************/
/*
int tapdelfir(int in,int ntaps, float garray[],int darray[],int *vecin,int maxtap,double *maxga)
{
 int out=0,i;
 double outfl=0;
 vecin[darray[0]]=in;
 for(i=0;i<ntaps;i++){
	outfl+=(double)((double)vecin[darray[i]]*scalef)*garray[i];
	}
 out=(int)(outfl*scale);
 for(i=0;i<ntaps;i++){
	if(--darray[i]<0)
	darray[i]=maxtap;
	}
 if(fabs(outfl)>*maxga){*maxga=fabs(outfl);}
 return(out);
}
*/
