/*simpleflt.c by Oscar Pablo Di Liscia*/

#include "../sndlib-l/io.c"
#include "../sndlib-l/headers.c"
#include "../sndlib-l/sound.c"
#include "../my_funcs/io_funcs.h"
#include "simpleflt.h"

int main(int argc, char *argv[])
{
  SOUND_DATA *in_data=NULL,*out_data=NULL;
  mus_sample_t **obuf, **ibuf;
  int i;
  int maxargs;

  void process(SOUND_DATA *in,SOUND_DATA *out, mus_sample_t **ibuf,mus_sample_t **obuf, float delay);
  
  maxargs=4;

  if(checkin(argc, maxargs, "Command line: sndelay infile outfile delay(segs.)", "SNDELAY")==-1) {
    getchar();
    return(-1);
  }

  mus_sound_initialize();
  
  if((in_data=open_input(argv[1],  "\nINPUT FILE:"))==NULL)
    {return(-1);}
  if((out_data=open_output(argv[2],"\nOUTPUT FILE:",in_data))==NULL)
    {return(-1);}
  
  /*RESERVAR MEMORIA*/
  ibuf=snd_memalloc(in_data, BUFFER_SIZE);
  obuf=snd_memalloc(out_data, BUFFER_SIZE);
  
  
  /*PROCESAR*/
  process(in_data, out_data, ibuf, obuf, atof(argv[3]));

  /*LIBERAR MEMORIA, CERRAR ARCHIVOS*/
  snd_dealloc(in_data, BUFFER_SIZE, ibuf);
  snd_dealloc(out_data, BUFFER_SIZE, obuf);
  close(in_data->fd);
  close(out_data->fd);
  free(in_data);
  free(out_data);

  return(0);
}

/***************************************************************/
void process(SOUND_DATA *in,SOUND_DATA *out, mus_sample_t **ibuf,mus_sample_t **obuf, float delay)
{
  int i,k,j;
  int size=BUFFER_SIZE;
  float insamp, outsamp;
  float maxgain=0.;
  float ant=0.;
  float ds1, ds2, ds3;
  int maxdelay;
  DDATA *ddata;
  //float delsamps=(float)in->srate*delay;
  //maxdelay=(int)delay+1;

  ds1=(float)in->srate*.1;
  ds2=(float)in->srate*.2;
  ds3=(float)in->srate*.3;
  maxdelay=(int)ds3;

  ddata=(DDATA*)set_delay(maxdelay+1);
  out->frames+=maxdelay;

  for (i=0; i < out->frames; i+=BUFFER_SIZE) {
      mus_sound_read (in->fd, 0,size-1,in->chans, ibuf);    
      for (k=0; k<in->chans; k++) {
      
	for (j=0; j<size; j++) {
	  if(i+j >= out->frames) break;
	  //if(i+j >= in->frames) 
	    //insamp=0.; ///necesario para los delays...
	  //else 
	    insamp=MUS_SAMPLE_TO_FLOAT(ibuf[k][j]);
	  w_delay(insamp, ddata);
	  outsamp =r_delay(ddata, ds1, 0);
	  outsamp+=r_delay(ddata, ds2, 1);
	  outsamp+=r_delay(ddata, ds3, 2);
	  //outsamp=fdelay(insamp, ddata, delsamps,0);
	  outsamp*=.25;
	  obuf[k][j]=MUS_FLOAT_TO_SAMPLE(outsamp);
	  maxgain=(outsamp > maxgain ? outsamp : maxgain);
	}

      }  
      if(j != BUFFER_SIZE && j !=0) size=j-1;
	mus_sound_write(out->fd,0,size-1,out->chans,obuf);
  }
  mus_sound_close_input (in->fd); 
  mus_sound_close_output(out->fd, out->frames * mus_bytes_per_sample(out->format));

  fprintf(stdout,"\n Maxgain= %f",maxgain);
  fprintf(stdout,"\n\n %ld frames of %ld samples each one written", ((i-BUFFER_SIZE)+j),out->chans );
  return;
}
/***************************************************************/
