#include "../sndlib-l/io.c"
#include "../sndlib-l/headers.c"
#include "../sndlib-l/sound.c"
#include "../my_funcs/io_funcs.h"

#define LP     0
#define HP     1
#define RECT   0
#define HAMM   1
#define NMAX   128

typedef struct {
  float re;
  float im;
}complex;

typedef struct {
  float input;
  float acoef[128];
  float ahist[128];
  int   N;
  int   now;
  int   j;
}FIR;

/*
Extensiones:
             otros tipos de filtros y ventanas
	     tomar respuesta impulso de un archivo de audio
	     respuesta a impulso con fases dispuestas para decorrelacion
*/


int main(int argc, char *argv[])
{
  SOUND_DATA *in_data=NULL,*out_data=NULL;
  mus_sample_t **obuf, **ibuf;
  int i;
  FIR *filter;

  
  void process(SOUND_DATA *in,SOUND_DATA *out, mus_sample_t **ibuf,mus_sample_t **obuf,FIR *filter, float gain);
  void set_freq(int type, int N, float CF, complex X[]);
  void set_phase(int type, int N, float CF, complex X[]);
  void polar_to_rect(int N, complex X[]);  
  void idft(float x[],int N, complex X[]);
  int  set_fir(int N, float F, int type, int window, SOUND_DATA *sdata, FIR *filter);
  float fir(FIR *filter);
 

  if(checkin(argc, 8, "Command line: firdes infile outfile  ncoef(>0<128) cutf(Hz) \n filtertype(0=Lowpass, 1=Hipass) smoothing window(0=rectangular, 1=Hamming) gain", "FIRDES")==-1) {
    getchar();
    return(-1);
  }


  mus_sound_initialize();
  
  if((in_data=open_input(argv[1],  "\nINPUT FILE:"))==NULL)
    {return(-1);}
  if((out_data=open_output(argv[2],"\nOUTPUT FILE:",in_data))==NULL)
    {return(-1);}
  
  /*RESERVAR MEMORIA*/
  ibuf=snd_memalloc(in_data, BUFFER_SIZE);
  obuf=snd_memalloc(out_data, BUFFER_SIZE);
  
  /*CONSTRUIR FILTRO*/
  filter=(FIR*)malloc(sizeof(FIR)*in_data->chans);
  if(set_fir(atoi(argv[3]),atof(argv[4]),atoi(argv[5]),atoi(argv[6]), in_data, filter)==0)
    {
      /*PROCESAR*/
      process(in_data, out_data, ibuf, obuf, filter, atof(argv[7]));
    }
  else {fprintf(stdout,"\n Program Aborted \n");}

/*LIBERAR MEMORIA, CERRAR ARCHIVOS*/
  snd_dealloc(in_data, BUFFER_SIZE, ibuf);
  snd_dealloc(out_data, BUFFER_SIZE, obuf);
  close(in_data->fd);
  close(out_data->fd);
  free(in_data);
  free(out_data);
  free(filter);
  
  return(0);
}

/*******************************************************************/
float fir(FIR *filter){

  int i;
  float output;
 
 filter->ahist[filter->now]=filter->input;
 output=0.0;
 filter->j=filter->now;
 
 //fprintf(stdout,"\nCOMIENZO now=%d=j=%d \n",filter->now,filter->j);
 //fprintf(stdout,  "LOOP     j= ");
 for (i=0; i< filter->N; i++){
   output +=  filter->acoef[i]* filter->ahist[filter->j++];
        //fprintf(stdout,"%d ",filter->j);
   if (filter->j >=filter->N)
     filter->j=0;
 }

 (filter->now)--;
 if (filter->now<0)
   filter->now=filter->N-1;
 //fprintf(stdout,"\nFINAL j=now=%d",filter->j);
 return(output);
}
/***************************************************************/
void process(SOUND_DATA *in,SOUND_DATA *out, mus_sample_t **ibuf,mus_sample_t **obuf, FIR *filter, float gain)
{
  int i,k,j;
  int size=BUFFER_SIZE;
  float isamp, osamp, maxamp=0.;

  //out->frames=8;

  for (i=0; i < out->frames; i+=BUFFER_SIZE) {
      mus_sound_read (in->fd, 0,size-1,in->chans, ibuf);    
      for (k=0; k<in->chans; k++) {

	for (j=0; j<size; j++) {
	  if(i+j == out->frames) break; 
	  filter[k].input=MUS_SAMPLE_TO_FLOAT(ibuf[k][j]);
	  osamp=fir(&filter[k])*gain;
	  obuf[k][j]=MUS_FLOAT_TO_SAMPLE(osamp);
	  maxamp=(osamp > maxamp? osamp: maxamp);
	}

      }  
      if(j != BUFFER_SIZE && j !=0) size=j-1;
	mus_sound_write(out->fd,0,size-1,out->chans,obuf);
  }
  mus_sound_close_input (in->fd); 
  mus_sound_close_output(out->fd, out->frames * mus_bytes_per_sample(out->format));

  fprintf(stdout,"\n\n MAXAMP= %3.3f \n %ld frames of %ld samples each one written", maxamp, ((i-BUFFER_SIZE)+j),out->chans );
  return;
}
/***********************************************************/
void set_freq(int type, int N, float CF, complex X[])
{
  int n;
  switch(type){
  case LP: 
    for(n=0; n<(N+1)/2; n++) {
      X[n].re= n <(int)((N-1)*CF) ? 1. : 0.;
      X[N-n].re=X[n].re;
    }
    break;
  case HP: 
    for(n=0; n<(N+1)/2; n++) {
      X[n].re= n >(int)((N-1)*CF) ? 1. : 0;
      X[N-n].re=X[n].re;
    }
    break;
	
  }

  return;
}
/***********************************************************/
void set_phase(int type, int N, float CF, complex X[])
{
  float alpha= -(N-1.)/2.;
  double PI2= 8.*atan(1.);
  int n;

  for(n=0; n<(N+1)/2; n++) {
    X[n].im= alpha*n*PI2/N;;
    X[N-n].im=-X[n].im;
  }
  return;
}
/*****************************************************************/
void polar_to_rect(int N, complex X[])
{
  float a, b;
  int n;

  /*convierto de polar a rectangular y divido por N para normalizar ganancia*/
  for (n=0; n<N; n++) {
    a= X[n].re*cos((double)X[n].im)/N;
    b= X[n].re*sin((double)X[n].im)/N;
    X[n].re=a;
    X[n].im=b;
  }
  return;
}
/*****************************************************************************/
void idft(float x[],int N, complex X[]){

double pi2oN=8.*atan(1.)/N;
int k,n;
 
 for (n=0; n<N; n++) {
   x[n]=0.0;
   for (k=0; k<N; k++) {
     x[n] += X[k].re*cos(pi2oN*k*n)- X[k].im*sin(pi2oN*k*n);
   }
 }

 return;
}
/*****************************************************************************/
int  set_fir(int N, float F, int type, int window, SOUND_DATA *sdata, FIR *filter)
{
  char *filtertype;
  char *windowtype;
  double CF;
  int i,n;
  complex X[NMAX];
  float h[NMAX];
  double PI2= 8.*atan(1.);

  if(type==LP) filtertype="Lowpass";
  else filtertype="Hipass";
  if(type != LP && type != HP) 
    {
      fprintf(stdout,"\n Unknown Filter Option (%d)", type);
      return(-1);
    }
  if(window==RECT) windowtype="Rectangular(non smoothing window)";
  else windowtype="Hamming";
  if(window != RECT && window != HAMM) 
    {
      fprintf(stdout,"\n Unknown Window Option (%d)", window);
      return(-1);
    }   
  fprintf(stdout,"\n Filter Length=\t%d        \n Cutoff Freq. =\t%5.3f", N, F);
  fprintf(stdout,"\n Filter Type  =\t%s        \n Window       =\t%s", filtertype, windowtype);

  CF= F / (double)sdata->srate;
  
  set_freq (type, N, CF, X);/*respuesta de frecuencia simetrica alrededor de cero*/
  set_phase(type, N, CF, X);/*fase con respuesta lineal*/
  polar_to_rect(N,X);
  idft(h,N,X);      /*trasformada de Fourier inversa*/
  if(window==HAMM) {/*ventana HAMMING*/
    for(n=0;n<N;n++){
      h[n]=h[n]*(.54-.46*cos(n*PI2/(N-1)));
    }
  }
  /*Preparar datos del filtro*/
  for(i=0; i<sdata->chans; ++i) {
    for(n=0;n<N;n++){
      filter[i].acoef[n]=h[n];
      filter[i].ahist[n]=0.;
    }
    filter[i].N=N;
    filter[i].now=0;
  }

  return(0);
}
/*******************************************************************/

