double	pii, pi2, p7, p7two, c22, s22;

/* LONGFFT.C
   all these code was borrowed  from the IEEE library
   and adapted for the use of convolve
*/

void cfast  (float *b,long int N);
void cfsst  (float *b,long int N);
void cfr2tr (long int off,float *b0,float  *b1);
void cford1 (int m,float  *b);
void cfr4syn(long int ofo,long int nn,float *b0,float  *b1,float *b2,float *b3,float  *b4,float *b5,float *b6,float *b7); 
void cfr4tr (long int off,long int nn,float  *b0,float  *b1,float *b2,float *b3,float *b4,float *b5,float  *b6,float  *b7);
void cford2 (int m,float  *b);

/*---------------------------------------
	cford1.cpp
in-place reordering subroutine
#--------------------------------------*/
void cford1(int m,float *b)

{
	long int k, kl, n, j;
	float t;
	k = 4;
	kl = 2;
	n = (long int)pow(2.,(float)m);
	for (j = 4; j <= n; j+=2) {
		if (k>j) {
			t = b[j-1];
			b[j-1] = b[k-1];
			b[k-1] = t;
		}
		k = k-2;
		if (k<=kl) {
			k = 2*j;
			kl = j;
			}
		}

	return;
}

 /*---------------------------
	cfr2tr.c
Radix 2 iteration subroutine.
---------------------------*/

void cfr2tr(long int off,float  *b0,float  *b1)

	{
	long int	i;
	float	tmp;
	for (i=0; i<off; i++)
	{
		tmp = b0[i] + b1[i];
		b1[i] = b0[i] - b1[i];
		b0[i] = tmp;
		}

	return;
}
/* -----------------------------------------------------------------
				cfast.c

This is the FFT subroutine fast.f from the IEEE library recoded in C.
	It takes a pointer to a real vector b[i], for i = 0, 1, ...,
	N-1, and replaces it with its finite discrete fourier 	transform.

-------------------------------------------------------------------*/

void cfast(float *b,long int N)


{
/* The DC term is returned in location b[0] with b[1] set to 0.
	Thereafter, the i'th harmonic is returned as a complex
	number stored as b[2*i] + j b[2*i+1].  The N/2 harmonic
	is returned in b[N] with b[N+1] set to 0.  Hence, b must
	be dimensioned to size N+2.  The subroutine is called as
	fast(b,N) where N=2**M and b is the real array described
	above.*/

	float	pi8,tmp;
	int	M;
	long int	i,Nt,off;

		pii = 4. * atan(1.);
		pi2 = pii * 2.;
		pi8 = pii / 8.;
		p7 = 1. / sqrt(2.);
		p7two = 2. * p7;
		c22 = cos(pi8);
		s22 = sin(pi8);
/* determine M */

	for (i=1, Nt=2; i<30; i++,Nt *= 2) if (Nt==N) break;
			M = i;
			if (M==30){
		fprintf(stderr,"fast: N is not an allowable power of 			two\\n");exit(1);
		}

/* do a radix 2 iteration first if one is required */

	Nt = 1;
	if (M%2 != 0){
		Nt = 2;
		off = N / Nt;
		cfr2tr(off,b,b+off);
	}
/* perform radix 4 iterations */
	if (M != 1) for (i=1; i<=M/2; i++){
		Nt *= 4;
		off = N / Nt;
		cfr4tr(off,Nt,b,b+off,b+2*off,b+3*off,b,b+off,b+2*off,b+3*off);
		}
/* perform in-place reordering */

	cford1(M,b);
	cford2(M,b);
	tmp = b[1];
	b[1] = 0.;
	b[N] = tmp;
	b[N+1] = 0.;

	for (i=3; i<N; i+=2) b[i] *= -1.;

	return;
}
/* -----------------------------------------------------------------\
				cfsst.c

This is the FFT subroutine fsst.f from the IEEE library recoded in C.
	It takes a pointer to a vector b[i], for i = 0, 1, ..., N-1,
	and replaces it with its inverse DFT assuming real output.

-------------------------------------------------------------------*/

void cfsst(float  *b,long int N)
	{

/* This subroutine synthesizes the real vector b[k] for k=0, 1,
	..., N-1 from the fourier coefficients stored in the b
	array of size N+2.  The DC term is in location b[0] with
	b[1] equal to 0.  The i'th harmonic is a complex number
	stored as b[2*i] + j b[2*i+1].  The N/2 harmonic is in
	b[N] with b[N+1] equal to 0. The subroutine is called as
	fsst(b,N) where N=2**M and b is the real array described
	above.
*/
	extern double	pii, pi2, p7, p7two, c22, s22;
	float	pi8,invN;
	int	M;
	long int	i,Nt,off;

	pii = 4. * atan(1.);
	pi2 = pii * 2.;	pi8 = pii / 8.;
	p7 = 1. / sqrt(2.);
	p7two = 2. * p7;
	c22 = cos(pi8);
	s22 = sin(pi8);
/* determine M */
	for (i=1, Nt=2; i<30; i++,Nt *= 2) if (Nt==N) break;
	M = i;
	if (M==30)
	{
		fprintf(stderr,"fast: N is not an allowable power of 	two\\n");
		exit(1);}

	b[1] = b[N];
	for (i=3; i<N; i+=2) b[i] *= -1.;

/* scale the input by N */

	invN = 1. / N;
	for (i=0; i<N; i++) b[i] *= invN;

/* scramble the inputs */

	cford2(M,b);
	cford1(M,b);

/* perform radix 4 iterations */

	Nt = 4*N;
	if (M != 1) for (i=1; i<=M/2; i++)
	{
		Nt /= 4;
		off = N / Nt;
		cfr4syn(off,Nt,b,b+off,b+2*off,b+3*off,b,b+off,b+2*off,b+3*off);
	}

/* do a radix 2 iteration if one is required */

	if (M%2 != 0)
	{		off = N / 2;
		cfr2tr(off,b,b+off);
	}
	return;
}

/*******************************************************************************/

void cfr4syn (long int ofo,long int nn,float *b0,float  *b1,float *b2,float *b3,float  *b4,float  *b5,float *b6,float *b7)
{
	double	pii, pi2,pi8, p7, p7two, c22, s22;

	float	piovn, arg, c1, c2, c3, s1, s2, s3,
		t0, t1, t2, t3, t4, t5, t6, t7;
	long int	i, off4, jj0, jlast, k0, kl, ji, jl, jr, j, k,
		jj1, j2, j3, j4, j5, j6, j7, j8, j9, j10, j11, j12,
		j13, j14, j15, j16, j17, j18, jthet, th2,
		l1, l2, l3, l4, l5, l6, l7, l8, l9, l10, l11, l12,
		l13, l14, l15, l16, l17, l18, l19,
		l[19];
		pii = 4. * atan(1.);
		pi2 = pii * 2.;
		pi8 = pii / 8.;
		p7 = 1. / sqrt(2.);
		p7two = 2. * p7;
		c22 = cos(pi8);
		s22 = sin(pi8);
/* jthet is a reversed binary counter; jr steps two at a time to
	locate real parts of intermediate results, and ji locates
	the imaginary part corresponding to jr.
*/

	l[0] = nn/4;
	for (i=1; i<19; i++)
	{
		if (l[i-1]<2) l[i-1] = 2;
		else if (l[i-1]!=2) 
	{
			l[i] = l[i-1]/2;
			continue;
		
	}
		l[i] = 2;
	
		}
	l1 = l[18];l2 = l[17];l3 = l[16];l4 = l[15];l5 = l[14];
	l6 = l[13];l7 = l[12];l8 = l[11];l9 = l[10];l10 = l[9];
	l11 = l[8];l12 = l[7];l13 = l[6];l14 = l[5];l15 = l[4];
	l16 = l[3];l17 = l[2];l18 = l[1];l19 = l[0];

	piovn = pii / nn;
	ji = 3;
	jl = 2;
	jr = 2;

	jj1 = 2;
loop1:
	 j2 = jj1;
loop2:
	j3 = j2;
loop3:
	j4 = j3;
loop4:	    
  j5 = j4;
loop5: 	     
		  for(j6=j5;j6<=l6; j6+=l5){
			for (j7=j6; j7<=l7; j7+=l6){
			 for (j8=j7; j8<=l8; j8+=l7){
			  for (j9=j8; j9<=l9; j9+=l8){
				for (j10=j9; j10<=l10; j10+=l9){
				 for (j11=j10; j11<=l11; j11+=l10){
				  for (j12=j11; j12<=l12; j12+=l11){
					for (j13=j12; j13<=l13; j13+=l12){
					 for (j14=j13; j14<=l14; j14+=l13){
					  for (j15=j14; j15<=l15; j15+=l14){
						for (j16=j15; j16<=l16; j16+=l15){
						 for (j17=j16; j17<=l17; j17+=l16){
						  for (j18=j17; j18<=l18; j18+=l17){
							for (jthet=j18; jthet<=l19; jthet+=l18){
							 th2 = jthet - 2;
							 if (th2 > 0){
			    arg = th2 * piovn;
			    c1 = cos(arg);
			    s1 = -sin(arg);
			    c2 = c1*c1 - s1*s1;
			    s2 = c1*s1 + c1*s1;
			    c3 = c1*c2 - s1*s2;
			    s3 = c2*s1 + s2*c1;
			    off4 = 4 * ofo;
			    jj0 = jr * off4;
			    k0 = ji * off4;
			    jlast = jj0 + ofo;
				 for (j=jj0; j<jlast; j++){
			     k = k0+j-jj0;
			     t0 = b0[j] + b6[k];
			     t1 = b7[k] - b1[j];
			     t2 = b0[j] - b6[k];
			     t3 = b7[k] + b1[j];
			     t4 = b2[j] + b4[k];
			     t5 = b5[k] - b3[j];
			     t6 = b5[k] + b3[j];
			     t7 = b4[k] - b2[j];
			     b0[j] = t0 + t4;
			     b4[k] = t1 + t5;
			     b1[j] = (t2+t6)*c1 - (t3+t7)*s1;
			     b5[k] = (t2+t6)*s1 + (t3+t7)*c1;
			     b2[j] = (t0-t4)*c2 - (t1-t5)*s2;
			     b6[k] = (t0-t4)*s2 + (t1-t5)*c2;
			     b3[j] = (t2-t6)*c3 - (t3-t7)*s3;
			     b7[k] = (t2-t6)*s3 + (t3-t7)*c3;
				  }
			    jr = jr + 2;
			    ji = ji - 2;
				 if (ji <= jl){
					 ji = 2*jr - 1;
					 jl = jr;
			    
				}
			}
				else{
			    for (i=0; i<ofo; i++){
			     t0 = b0[i] + b1[i];
			     t1 = b0[i] - b1[i];
			     t2 = b2[i] * 2.0;
			     t3 = b3[i] * 2.0;
			     b0[i] = t0 + t2;
			     b2[i] = t0 - t2;
			     b1[i] = t1 + t3;
			     b3[i] = t1 - t3;
				  }
				 if (nn>4){
				  k0 = 4*ofo;
				  kl = k0 + ofo;
				  for (i=k0; i<kl; i++){
					t2 = b0[i] - b2[i];
					t3 = b1[i] + b3[i];
					b0[i] = (b0[i] + b2[i]) * 2.0;
					b2[i] = (b3[i] - b1[i]) * 2.0;
					b1[i] = (t2 + t3) * p7two;
					b3[i] = (t3 - t2) * p7two;
							}
						  }
						 }
						}
					  }
					 }
					}
				  }
				 }
				}
			  }
			 }
			}
		  }
		 }
		}
     }
	    j5 += l4;
		 if (j5 <= l5) goto loop5;
		 j4 += l3;
		 if (j4 <= l4) goto loop4;
		 j3 += l2;
		 if (j3 <= l3) goto loop3;
		 j2 += l1;
		 if (j2 <= l2) goto loop2;
		 jj1 += 2;
		 if (jj1 <= l1) goto loop1;
		 
return;
}
/*********************************************************************************************/
/*---------------------------
	cfr4tr.cpp

Radix 4 iteration subroutine.
---------------------------*/

void cfr4tr(long int off,long int nn,float  *b0,float  *b1,float *b2,float *b3,float *b4,float  *b5,float  *b6,float  *b7)

	
	{
	double	pii, pi2,pi8,p7,p7two, c22, s22;

	long int	i, off4, jj0, jlast, k0, kl, ji, jl, jr, j, k,
		jj1=0, j2, j3, j4, j5, j6, j7, j8, j9, j10, j11, j12,
		j13, j14, j15, j16, j17, j18, jthet, th2,
		l1, l2, l3, l4, l5, l6, l7, l8, l9, l10, l11, l12,
		l13, l14, l15, l16, l17, l18, l19,  l[19];

	float	piovn, arg, c1, c2, c3, s1, s2, s3, r1, r5, pr, pi,
		t0, t1, t2, t3, t4, t5, t6, t7;

		pii = 4. * atan(1.);
		pi2 = pii * 2.;
		pi8 = pii / 8.;
		p7 = 1. / sqrt(2.);
		p7two = 2. * p7;
		c22 = cos(pi8);
		s22 = sin(pi8);
/* jthet is a reversed binary counter; jr steps two at a time to
	locate real parts of intermediate results, and ji locates
	the imaginary part corresponding to jr.
*/

	l[0] = nn / 4;
	for (i=1; i<19; i++)
	{
		if (l[i-1] < 2) l[i-1] = 2;
		else if (l[i-1] != 2){
		l[i] = l[i-1] / 2;
			continue;
		}
		l[i] = 2;
		}

	l1 = l[18];l2 = l[17];l3 = l[16];l4 = l[15];l5 = l[14];l6 = l[13];
	l7 = l[12];l8 = l[11];l9 = l[10];l10 = l[9];l11 = l[8];l12 = l[7];
	l13 = l[6];l14 = l[5];l15 = l[4];l16 = l[3];l17 = l[2];l18 = l[1];
	l19 = l[0];
	piovn = pii / nn;
	ji = 3;jl = 2;jr = 2;

	  jj1 = 2;
loop1:
	  j2 = jj1;
loop2:
	  j3 = j2;
loop3:
	  j4 = j3;
loop4:	    
	  j5 = j4;
loop5:
		  for(j6=j5;j6<=l6; j6+=l5){
			for (j7=j6; j7<=l7; j7+=l6){
			 for (j8=j7; j8<=l8; j8+=l7){
			  for (j9=j8; j9<=l9; j9+=l8){
				for (j10=j9; j10<=l10; j10+=l9){
				 for (j11=j10; j11<=l11; j11+=l10){
				  for (j12=j11; j12<=l12; j12+=l11){
					for (j13=j12; j13<=l13; j13+=l12){
					 for (j14=j13; j14<=l14; j14+=l13){
					  for (j15=j14; j15<=l15; j15+=l14){
						for (j16=j15; j16<=l16; j16+=l15){
						 for (j17=j16; j17<=l17; j17+=l16){
						  for (j18=j17; j18<=l18; j18+=l17){
							for (jthet=j18; jthet<=l19; jthet+=l18){
							 th2 = jthet - 2;
            if (th2 <= 0){
				 for (i=0; i<off; i++){
			     t0 = b0[i] + b2[i];
			     t1 = b1[i] + b3[i];
			     b2[i] = b0[i] - b2[i];
			     b3[i] = b1[i] - b3[i];
			     b0[i] = t0 + t1;
			     b1[i] = t0 - t1;
				  }
				 if (nn > 4){
			     k0 = 4 * off;
			     kl = k0 + off;
				  for (i=k0; i<kl; i++){
			      pr = p7 * (b1[i] - b3[i]);
			      pi = p7 * (b1[i] + b3[i]);
			      b3[i] = b2[i] + pi;
			      b1[i] = pi - b2[i];
			      b2[i] = b0[i] - pr;
					b0[i] = b0[i] + pr;
					}
				  }
				}
				else{
			    arg = th2 * piovn;
			    c1 = cos(arg);
			    s1 = sin(arg);
			    c2 = c1*c1 - s1*s1;
			    s2 = c1*s1 + c1*s1;
			    c3 = c1*c2 - s1*s2;
			    s3 = c2*s1 + s2*c1;
			    off4 = 4 * off;
			    jj0 = jr * off4;
			    k0 = ji * off4;
			    jlast = jj0 + off;
				 for (j=jj0; j<jlast; j++){
			     k = k0 + j - jj0;
			     r1 = b1[j]*c1 - b5[k]*s1;
			     r5 = b1[j]*s1 + b5[k]*c1;
			     t2 = b2[j]*c2 - b6[k]*s2;
			     t6 = b2[j]*s2 + b6[k]*c2;
			     t3 = b3[j]*c3 - b7[k]*s3;
			     t7 = b3[j]*s3 + b7[k]*c3;
			     t0 = b0[j]+t2;
			     t4 = b4[k]+t6;
			     t2 = b0[j]-t2;
			     t6 = b4[k]-t6;
			     t1 = r1+t3;
			     t5 = r5+t7;
			     t3 = r1-t3;
			     t7 = r5-t7;
			     b0[j] = t0+t1;
			     b7[k] = t4+t5;
			     b6[k] = t0-t1;
			     b1[j] = t5-t4;
			     b2[j] = t2-t7;
			     b5[k] = t6+t3;
			     b4[k] = t2+t7;
			     b3[j] = t3-t6;
				 }
			    jr += 2;
			    ji -= 2;
			    if (ji <= jl){
			     ji = 2*jr - 1;
			     jl = jr;
						  }
						 }
						}
					  }
					 }
					}
				  }
				 }
				}
			  }
			 }
			}
		  }
		 }
		}
     }
	    j5 += l4;
		 if (j5 <= l5) goto loop5;
		 j4 += l3;
		 if (j4 <= l4) goto loop4;
				 j3 += l2;
		 if (j3 <= l3) goto loop3;
		 j2 += l1;
		 if (j2 <= l2) goto loop2;
		 jj1 += 2;
		 if (jj1 <= l1) goto loop1;
	 
return;
}

/*********************************************************************************************/

/*-------------------------------
	cford2.c
in-place reordering subroutine
-------------------------------*/
void cford2(int m,float  *b)
{
	float	t;
	long int	n, ij, ji, k,
		jj1, j2, j3, j4, j5, j6, j7, j8, j9, j10, j11, j12,
		j13, j14, j15, j16, j17, j18, 
		l1, l2, l3, l4, l5, l6, l7, l8, l9, l10, l11, l12,
		l13, l14, l15, l16, l17, l18, l19,  l[19];

	n =(long int) pow(2.,(float)m);
	l[0] = n;
	for (k = 2; k <= m; k++) l[k-1] = l[k-2]/2;
	for (k = m; k <= 18; k++) l[k] = 2;
	ij = 2;
   l1 = l[18];l2 = l[17];l3 = l[16];l4 = l[15];l5 = l[14];
	l6 = l[13];l7 = l[12];l8 = l[11];l9 = l[10];l10 = l[9];
	l11 = l[8];l12 = l[7];l13 = l[6];l14 = l[5];l15 = l[4];
	l16 = l[3];l17 = l[2];l18 = l[1];l19 = l[0];


	jj1 = 2;
loop1:
	j2 = jj1;
loop2:
	j3 = j2;
loop3:
	j4 = j3;
loop4:	    
   j5 = j4;
loop5:
		  for(j6=j5;j6<=l6; j6+=l5){
			for (j7=j6; j7<=l7; j7+=l6){
			 for (j8=j7; j8<=l8; j8+=l7){
			  for (j9=j8; j9<=l9; j9+=l8){
				for (j10=j9; j10<=l10; j10+=l9){
				 for (j11=j10; j11<=l11; j11+=l10){
				  for (j12=j11; j12<=l12; j12+=l11){
					for (j13=j12; j13<=l13; j13+=l12){
					 for (j14=j13; j14<=l14; j14+=l13){
					  for (j15=j14; j15<=l15; j15+=l14){
						for (j16=j15; j16<=l16; j16+=l15){
						 for (j17=j16; j17<=l17; j17+=l16){
						  for (j18=j17; j18<=l18; j18+=l17){
	
			  for (ji=j18; ji<=l19; ji+=l18){
		 	   if (ij<ji){
			    t = b[ij-2];
			    b[ij-2] = b[ji-2];
			    b[ji-2] = t;
			    t = b[ij-1];
			    b[ij-1] = b[ji-1];
			    b[ji-1] = t;
			    }
			   ij = ij+2;
			   		}
					  }
					 }
					}
				  }
				 }
				}
			  }
			 }
			}
		  }
		 }
		}
     }
	    j5 += l4;
		 if (j5 <= l5) goto loop5;
		 j4 += l3;
		 if (j4 <= l4) goto loop4;
		 j3 += l2;
		 if (j3 <= l3) goto loop3;
		 j2 += l1;
		 if (j2 <= l2) goto loop2;
		 jj1 += 2;
		 if (jj1 <= l1) goto loop1;
	 
return;
}


