/*io_funcs.h*/

typedef struct {
  int fd;
  int chans;
  int srate;
  off_t samples;
  off_t frames;
  float length;
  int format;
  int header;
  char *comment;
}SOUND_DATA;

#define BUFFER_SIZE 1000  /*IO Buffer size in samples*/

int	     checkin(int args, int nargs, char *msj, char *exename); 
void         init_sound_data(SOUND_DATA *s, char *filename);
void 	     print_sound_data(char *filename, SOUND_DATA *d, char *msj);
mus_sample_t **snd_memalloc(SOUND_DATA *sdata, long int size);
void         snd_dealloc(SOUND_DATA *sdata, long int size, mus_sample_t **buf);
SOUND_DATA   *open_input(char* name, char* msj);
SOUND_DATA   *open_output(char* name, char* msj, SOUND_DATA *data);



/***************************************************************/
int checkin(int args, int nargs, char *msj, char *exename)
{
  if(args < nargs) {
    fprintf(stderr, "\n  %s: insufficient arguments[%d] \n  %s",exename, args, msj);
    fprintf(stdout,"\n\n Press return to terminate program..."); 
    return(-1);
  }
  return(0);
}
/***************************************************************/
void init_sound_data(SOUND_DATA *d, char *filename)
{
  d->srate   = mus_sound_srate(filename);
  d->chans   = mus_sound_chans(filename);
  d->samples = mus_sound_samples(filename);
  d->frames  = mus_sound_frames(filename); /*frames are NOT the same as samples...!*/
  d->comment = mus_sound_comment(filename); 
  d->format  = mus_sound_data_format(filename);
  d->header  = mus_sound_header_type(filename);
  d->length = (double)d->frames / (float)d->srate;
  return;
}
/***************************************************************/
void print_sound_data(char *filename, SOUND_DATA *d, char *msj)
{
  
  fprintf(stdout, "\n%s\t%s:\n  SR: \t\t%d\n  CHANNELS: \t%d\n  LENGTH: \t%f\n  SAMPLES: \t%d\n  FRAMES: \t%d\n" ,msj, filename, d->srate, d->chans, d->length,d->samples,d->frames);/*frames are NOT the same as samples...!*/
      
  fprintf(stdout, "  TYPE: \t%s\n  FORMAT: \t%s\n  COMMENT: \t%s\n", 
	  mus_header_type_name(mus_sound_header_type(filename)), 
	  mus_data_format_name(mus_sound_data_format(filename)),d->comment);

  return;
}
/***************************************************************/
mus_sample_t **snd_memalloc(SOUND_DATA *sdata, long int size)
{
  mus_sample_t **buf;
  int i;

  buf = (mus_sample_t **)CALLOC(sdata->chans,sizeof(int *));
  for (i=0; i < sdata->chans; i++) buf[i] = (mus_sample_t *)CALLOC(size,sizeof(mus_sample_t));

  return(buf);
}
/***************************************************************/
void snd_dealloc(SOUND_DATA *sdata, long int size, mus_sample_t **buf)
{
  int i;
  for (i=0; i < sdata->chans; i++) FREE(buf[i]);
  FREE(buf);
  return;

}
/***************************************************************/
SOUND_DATA *open_input(char* name, char* msj)
{
  SOUND_DATA *sdata;
  
  sdata=(SOUND_DATA*)malloc(sizeof(SOUND_DATA));
  sdata->fd  = mus_sound_open_input(name);
  if (sdata->fd != -1)
    {
      init_sound_data(sdata,name);
      print_sound_data(name,sdata, msj);  
    }
  else {
    fprintf(stderr, "%s: %s\n", name, strerror(errno));
    fprintf(stdout,"\n\n Press return to terminate program..."); 
    //getchar();
    return(NULL);
  }
  return(sdata);
}
/***************************************************************/
SOUND_DATA *open_output(char* name, char* msj, SOUND_DATA *data)
{
  SOUND_DATA *odata;

  odata= (SOUND_DATA*)malloc(sizeof(SOUND_DATA));
  odata->fd = mus_sound_open_output(name,data->srate, 
				       data->chans, 
				       data->format, 
				       data->header,
				       "created by sndili");
  if (odata->fd != -1)
    {
      odata->srate= data->srate;
      odata->chans= data->chans;
      odata->format= data->format;
      odata->header=data->header;
      odata->frames=data->frames;
      odata->samples=data->samples;
      odata->comment="created by sndili";
      odata->length = (double)odata->samples / (float)(odata->chans * odata->srate);

      print_sound_data(name,odata, msj);  
    }
  else {
    fprintf(stderr, "%s: %s\n",name, strerror(errno));
    fprintf(stdout,"\n\n Press return to terminate program..."); 
    getchar();
    return(NULL);
  }
  return(odata);
}
