/*ug_funcs.h by Pablo Di Liscia*/

#include <math.h>

double PI = M_PI;
double PI2= 2. * M_PI;


#define MIN_FLOAT (1./pow(2.,31.))

/*calcular incremento del oscilador a partir de la frecuencia*/ 
#define ITOF(frec, L, sr)  frec * ((float)L/(float)sr)

typedef struct {
  double  index;
  double  incr;
  double  phase;
}OSCIL;

typedef struct { //the data for the randi UG
  float size; //size of the frame in samples this should be sr/freq.
  float a1;   //current amplitude value
  float a2;   //next    amplitude value
  float cnt;  //sample  position counter
} RANDI;

typedef struct { 
  //ojo= trans recibe duraciones en proporcion de su largo total!
  int   size;          //Total samples to process
  int   nbp;          //<128
  float startval; //first value
  float val    [128];  //value (val[0] should be 0.)
  int   samps[128]; //time between values in samples
  float curve[128];//type of transition
  int   cnt;      //sample  position counter
  int   scnt;    //segment counter
  int   sizecont;
} TRANS;

float *make_sine_table(float amp, float phase, int length);
float *make_table_from_snd(SOUND_DATA *sdata);
OSCIL *set_ioscilator(int L, float phase);
float ioscilator(OSCIL *oscil, float frec, float amp, float *table, int L, int sr);
float sndoscil(OSCIL *oscil, float incr, float amp, float *table, int L);
RANDI *set_randi(int sr, float freq);
float randi(RANDI *radat, float freq, int sr);
TRANS *set_trans(int nsamps, int nbp, float transargs[]);


/********************************************************/
float *make_sine_table(float amp, float phase, int length)
{
  float *tp=NULL;
  int i;
  float theta=0.;
  float incr = PI2 / ((float)(length)+1);
  
  
  tp= (float *) malloc(length * sizeof(float)+1);
  
  theta+= phase * PI2; /*la fase se indica de 0 a 1*/
  
  for(i=0; i < length; i++) {
    tp[i] = sin(theta)*amp;
    theta +=incr;
  }
  
  return(tp);
}
/********************************************************/
float *make_table_from_snd(SOUND_DATA *sdata)
{
  /*se puede hacer mejor agregando el canal a leer
    e indicadores de start y end para el archivo*/
  
  float *tp=NULL;
  mus_sample_t **buf;
  int i, j;
  
  tp     = (float *) malloc(sdata->frames * sizeof(float)+1);
  buf   = (mus_sample_t **) malloc(sizeof(mus_sample_t*));
  buf[0]= (mus_sample_t *) malloc(BUFFER_SIZE * sizeof(mus_sample_t)+1);
  
  /*cargamos solo el canal 1 del archivo en memoria*/
  for (i=0; i < sdata->frames; i+=BUFFER_SIZE) {
    mus_sound_read (sdata->fd, 0, BUFFER_SIZE-1,1, buf);  
    for (j=0; j<BUFFER_SIZE; j++) {
      tp[i+j]=MUS_SAMPLE_TO_FLOAT(buf[0][j]);
    }
  }
  
  free(buf[0]);
  free(buf);
  return(tp);
}
/********************************************************/
OSCIL *set_ioscilator(int L, float phase)
{
  OSCIL *oscil;

  oscil=(OSCIL*)malloc(sizeof(oscil));

  oscil->incr =0.;
  oscil->index=(double)(phase*(double)L);
  
  return(oscil);
}
/********************************************************/
float ioscilator(OSCIL *oscil, float frec, float amp, float *table, int L, int sr)
{
  float output;
  int   v2=0;
  //Convertir a incremento la frecuencia
  oscil->incr= (double)frec * ((double)L/(double)sr);
  
  v2=(int)(oscil->index + 1.);
  while ( v2 >= L ) {v2 -= L;}
  
  //Interpolacion Lineal
    output=amp*(table[(int)oscil->index] + 	
	 ((table[v2] - table[(int)oscil->index]) * ( oscil->index - (int)oscil->index)));

  oscil->index  += oscil->incr;
  
  while ( oscil->index >= (float)L) {
    oscil->index -=(double)L;
  }
  while ( oscil->index < 0.) {
    oscil->index +=(double)L;
  }
  
  return(output);
}
/********************************************************/
float sndoscil(OSCIL *oscil, float incr, float amp, float *table, int L)
{
  /*ojo: sndoscil recibe solo incremento, no frecuencia*/
  /*es la unica diferencia con oscil*/
 
 float output;
  int   v2=0;
  
  //tautologico, pero necesario
  oscil->incr= (double)incr;

  v2=(int)(oscil->index + 1.);
  while ( v2 >= L ) {v2 -= L;}
 

  //Con Interpolacion Lineal
  output=amp*(table[(int)oscil->index] + 	
         ((table[v2] - table[(int)oscil->index]) * ( oscil->index - (int)oscil->index)));

  //Sin interpolacion      
  //output= amp * table[(int)oscil->index];

  oscil->index  += oscil->incr;
  
  while ( oscil->index >= (float)L) {
    oscil->index -=(double)L;
  }
  while ( oscil->index < 0.) {
    oscil->index +=(double)L;
  }
  
  return(output);
}


/********************************************************/
RANDI *set_randi(int sr, float freq)
{
  RANDI *randi;

  randi=(RANDI*)malloc(sizeof(RANDI));

  randi->size= (float)((float)sr / freq);
  randi->a1  = (float)rand();
  randi->a2  = (float)rand();
  randi->cnt =  0.;

  return(randi);
}
/******************************************************************/
//randi outputs random numbers in the range of 0-1
//getting a new number at frequency freq and interpolating
//the intermediate values.
float randi(RANDI *radat, float freq, int sr)
{
  float output, incr;
  
  if(radat->cnt >= radat->size) { //get a new random value
    radat->a1  = radat->a2; 
    radat->a2  = rand();
    radat->cnt = 0.;
    radat->size= (float) ((float)sr / freq);
  }

  output=(float)(((radat->a2 - radat->a1)/ radat->size) * radat->cnt)+ radat->a1;
  
  incr=(float)freq * ((radat->size)/(float)sr);
  radat->cnt+=incr;
  

  return(output/RAND_MAX);
}
/********************************************************************/
TRANS *set_trans(int nsamps, int nbp, float transargs[])
{
  TRANS *trans=NULL;
  int i;
  
  if(nbp > 127)
    {fprintf(stdout,"\n TRANS= too many breakpoints (%d) max. allowed=127",nbp); return(NULL);}

  trans=(TRANS*)malloc(sizeof(TRANS));
 
  trans->size=nsamps;
  trans->nbp =nbp;
  
  /*cargar valores*/
  trans->val[0]    =transargs[0];  
  //ojo= trans recibe duraciones en proporcion de su largo total!
  for(i=1; i<nbp+1; ++i) {
    trans->samps[i-1]  =(int)(floor(transargs[((i-1)*3)+1]*(float)trans->size));   

    trans->curve[i-1]=transargs[((i-1))*3+2]; 
    if(trans->curve[i-1]==0.) trans->curve[i-1]=MIN_FLOAT;

    trans->val  [i]=transargs[((i-1)*3)+3]; 
  }
  
  trans->cnt =0;
  trans->scnt=0;
  trans->sizecont=0;  

  return(trans);
}
/********************************************************************/
float trans(TRANS *trans)
{
  float out;
  double divisor, divi, tr;

  if(++trans->sizecont == trans->size){
    return(0.);
  }
  
  if(trans->val[trans->scnt]==trans->val[trans->scnt+1]){
    out=trans->val[trans->scnt];
  }
  else {
    tr     = (double)trans->curve[trans->scnt] / (double)(trans->samps[trans->scnt]-1);
    divisor= 1. - exp(trans->curve[trans->scnt]);
    divi   = 1. -(double)exp((double)trans->cnt*tr);
    
    out=(double)(trans->val[trans->scnt]+(double)(trans->val[trans->scnt+1]-trans->val[trans->scnt])*
		 (double)((divi/divisor)));
    
  }
  if(++trans->cnt == trans->samps[trans->scnt]-1) {
    trans->cnt=0;
    ++trans->scnt;
  }

  return(out);
}
/********************************************************************/
