/*sndiag.h*/
typedef struct {
  float linmax;
  float linmin;
  float dBmax;
  float dBmin;
  long int maxsamp;
  long int minsamp;
  /*RMS Variables*/
  long int cont;
  float sum; 
  float max_RMS;  
  long int  block_MAX_RMS;
  /*ZERO CROSSING VARIABLES*/
  float zant;
  long int zcont;
  long int zant_number;
  long int zsum ;
  /*POSSIBLY CLIPPED SAMPLES COUNTER*/
  long int pcs;
  float cant;
  /*DC OFFSET*/
  double posamp;
  double negamp;
  double poscont;
  double negcont;
  double posav;
  double negav;
}CHANNEL_DATA;

void 	print_report(SOUND_DATA *in, CHANNEL_DATA *cdata);
void 	do_analysis(SOUND_DATA *in, CHANNEL_DATA *cdata, mus_sample_t **bufs, long int RMSblock);
float   lintodB(float lin);
void    do_rms(float in, long int RMSblock, CHANNEL_DATA *cdata, long int sample_number);
void    do_zeroc(float in, CHANNEL_DATA *cdata, long int sample_number);
void    do_clip(float in, CHANNEL_DATA *cdata);
void    do_DC_AV(float in, CHANNEL_DATA *cdata);
void    do_amp(float in, CHANNEL_DATA *cdata, long int sample_number);

/***************************************************************/
void 	print_report(SOUND_DATA *in, CHANNEL_DATA *cdata)
{
  int k;

  for(k=0; k<in->chans; k++){
    fprintf(stdout,"\nREPORT OF CHANNEL %d:",k+1);
    if(cdata[k].linmax==0. && cdata[k].linmin==0.)
      fprintf(stdout,"\n  ALL ZERO SAMPLES ON THIS CHANNEL...!!!\n");
    else
      {
	fprintf(stdout,"\n   MAXAMP         = %1.3f (%2.3f dB)",cdata[k].linmax, lintodB(cdata[k].linmax));
	fprintf(stdout,"\t   AT SAMPLE = %ld (%5.3f secs.) ",cdata[k].maxsamp,(float)cdata[k].maxsamp/(float)in->srate );
	fprintf(stdout,"\n   MINAMP         = %1.3f (%2.3f dB)",cdata[k].linmin, lintodB(fabs(cdata[k].linmin)));
	fprintf(stdout,"\t   AT SAMPLE = %ld (%5.3f secs.) ",cdata[k].minsamp,(float)cdata[k].minsamp/(float)in->srate );
	fprintf(stdout,"\n   MAX RMS        = %1.3f (%2.3f dB)",cdata[k].max_RMS, lintodB(cdata[k].max_RMS));
	fprintf(stdout,"\t   AT SAMPLE = %ld (%5.3f secs.) ",cdata[k].block_MAX_RMS,
		(float)cdata[k].block_MAX_RMS/(float)in->srate );
	if(cdata[k].zcont !=0 || cdata[k].linmax !=0.){
	  fprintf(stdout,"\n   ZERO CROSSINGS = %5.3lf Hz.",
		  (double)in->srate/(double)(cdata[k].zsum / cdata[k].zcont));
	}
	else {
	  fprintf(stdout,"\n   NO ZERO CROSSINGS DETECTED ON THIS CHANNEL");
	}
	fprintf(stdout,"\n   POSSIBLY CLIPPED SAMPLES= %ld",cdata[k].pcs); 
	fprintf(stdout,"\n   D.C. OFFSET AVERAGE     = %3.1f%%",100.*((cdata[k].posav-cdata[k].negav)));
	
	fprintf(stdout,"\n ");
      }
  }
  
  return;
}
/***************************************************************/
void 	do_analysis(SOUND_DATA *in, CHANNEL_DATA *cdata, mus_sample_t **bufs, long int RMSblock)
{
  int i,k,j;
  float sample;
 
  for (i=0; i < in->frames; i+=BUFFER_SIZE) {

    mus_sound_read(in->fd,0,BUFFER_SIZE-1,in->chans,bufs);

    for (k=0; k<in->chans; k++) {

      for (j=0; j<BUFFER_SIZE; j++) {

	if(i+j > in->frames) break; /*because the buffer may be inaccurate*/
	sample=MUS_SAMPLE_TO_FLOAT(bufs[k][j]);

	do_rms(sample, RMSblock, &cdata[k], i+j);
	do_zeroc(sample, &cdata[k], i+j);
	do_DC_AV(sample, &cdata[k]);
	do_clip(sample, &cdata[k]);
	do_amp(sample, &cdata[k], i+j); 
      }
    }
  }

  for (k=0; k<in->chans; k++){
    if(cdata[k].poscont > 0)
      cdata[k].posav=cdata[k].posamp/cdata[k].poscont;
    if(cdata[k].negcont > 0)
      cdata[k].negav=cdata[k].negamp/cdata[k].negcont;
  }

  return;
}
/***************************************************************/
float lintodB(float lin)
{
  return((float)(20. * log10(lin)));
}

/***************************************************************/
void do_rms    (float in, long int RMSblock, CHANNEL_DATA *cdata, long int sample_number)
{
  cdata->sum+=(in * in);

  if(++cdata->cont == RMSblock-1)
    {
      cdata->cont=0;
      cdata->sum /= RMSblock;
      cdata->sum=(float)(sqrt((double)cdata->sum));
      if(cdata->sum > cdata->max_RMS) {
	cdata->max_RMS= cdata->sum;
	cdata->block_MAX_RMS=sample_number;
      }
      
    }  
  return;
}
/***************************************************************/
void do_zeroc(float in, CHANNEL_DATA *cdata, long int sample_number)
{
  if(cdata->zant*in < 0. || in == 0.) {
    cdata->zsum +=(sample_number - cdata->zant_number); 
    cdata->zant_number = sample_number;
    ++cdata->zcont;
  }
  cdata->zant=in; 
  return;
}
/***************************************************************/
void do_clip(float in, CHANNEL_DATA *cdata)
{

 if (in==cdata->cant)
    ++cdata->pcs;

 cdata->cant=in;
  return;
}
/***************************************************************/
void do_DC_AV(float in, CHANNEL_DATA *cdata)
{
  if(in >= 0.) {
    cdata->posamp += in;
    ++cdata->poscont;
  }
  if(in < 0.) {
    cdata->negamp += fabs(in);
    ++cdata->negcont;
  }

  return;
}
/***************************************************************/
void do_amp(float in, CHANNEL_DATA *cdata, long int sample_number)
{
  if(in > cdata->linmax) {
    cdata->linmax=in;
    cdata->maxsamp=sample_number;
  }
  if(in < cdata->linmin) {
    cdata->linmin=in;
    cdata->minsamp=sample_number;;
  }
  return;
}
/***************************************************************/
