#include "../sndlib-l/io.c"
#include "../sndlib-l/headers.c"
#include "../sndlib-l/sound.c"
#include "../my_funcs/io_funcs.h"
#include "../my_funcs/ug_funcs.h"

int main(int argc, char *argv[])
{
  SOUND_DATA *in_data=NULL,*out_data=NULL;
  mus_sample_t **obuf, **ibuf;
  int i;

  void process(SOUND_DATA *in,SOUND_DATA *out, mus_sample_t **ibuf,mus_sample_t **obuf , float semitonos);
    
  if(checkin(argc, 4, "Command line: foscil infile outfile semitones", "FOSCIL")==-1) {
    getchar();
    return(-1);
  }

  mus_sound_initialize();
  
  if((in_data=open_input(argv[1],  "\nINPUT FILE:"))==NULL)
    {return(-1);}
  if((out_data=open_output(argv[2],"\nOUTPUT FILE:",in_data))==NULL)
    {return(-1);}
  
  /*RESERVAR MEMORIA*/
  ibuf=snd_memalloc(in_data, BUFFER_SIZE);
  obuf=snd_memalloc(out_data, BUFFER_SIZE);
  
  /*PROCESAR*/
  process(in_data, out_data, ibuf, obuf, atof(argv[3]));

  /*LIBERAR MEMORIA, CERRAR ARCHIVOS*/
  snd_dealloc(in_data, BUFFER_SIZE, ibuf);
  snd_dealloc(out_data, BUFFER_SIZE, obuf);
  close(in_data->fd);
  close(out_data->fd);
  free(in_data);
  free(out_data);
  
  return(0);
}

/***************************************************************/
void process(SOUND_DATA *in,SOUND_DATA *out, mus_sample_t **ibuf,mus_sample_t **obuf, float semitonos)
{
  /*esta macro calcula un incremento de s semitonos*/
#define S_TO_INCR(s) (pow(2.,(double)s/12.))
  
  int i,k,j;
  int size=BUFFER_SIZE;
  float outsamp;
  float maxamp=0.;
  float *sndtable=NULL;
  float *sinetable=NULL;
  OSCIL *soscil, *oscil;
  float amp=1., incr, modout;
  float fmod, im;
  int otl=10000;


  incr=(float)S_TO_INCR(semitonos);
  /*para vibrato*/
  im= 1. - (float)S_TO_INCR(0.5); /*vibrato de un semitono en total*/
  fmod= 2.;                      /*a 2. Hz */
  /*para FM*/
  //fmod= 440.;  
  //im  = .01 * 440.; /* DeltaF=IM*Fm */

  out->frames =(int)((double)out->frames/S_TO_INCR(semitonos));   
  /*acortamos o alargamos el archivo de salida*/

  soscil     =(OSCIL*)set_ioscilator(in->frames, 0.);
  sndtable   =(float*)make_table_from_snd(in);
  oscil      =(OSCIL*)set_ioscilator(otl, 0.);
  sinetable  =(float*)make_sine_table(1.,0.,otl);

  /*ahora, usamos a sndtable como tabla para un oscilador*/
  for (i=0; i < out->frames; i+=BUFFER_SIZE) {
    for (k=0; k<in->chans; k++) {
      for (j=0; j<size; j++) {
	if(i+j == out->frames) break;
	modout =ioscilator(oscil,fmod,im,sinetable,otl,in->srate);
	outsamp=sndoscil(soscil,incr+modout,amp,sndtable,in->frames);
	obuf[k][j]=MUS_FLOAT_TO_SAMPLE(outsamp);
	maxamp=(outsamp > maxamp? outsamp : maxamp);
      }
    }  
    if(j != BUFFER_SIZE && j !=0) size=j-1;
    mus_sound_write(out->fd,0,size-1,out->chans,obuf);
  }

  mus_sound_close_input (in->fd); 
  mus_sound_close_output(out->fd, out->frames * mus_bytes_per_sample(out->format));

  fprintf(stdout,"\n\n MAXAMP=%5.3f", maxamp);
  fprintf(stdout,"\n\n %ld frames of %ld samples each one written", ((i-BUFFER_SIZE)+j),out->chans );

  free(sndtable);
  free(soscil);
  free(sinetable);
  free(oscil);

  return;
}
/***************************************************************/
