#include "../sndlib-l/io.c"
#include "../sndlib-l/headers.c"
#include "../sndlib-l/sound.c"
#include "../my_funcs/io_funcs.h"
#include "../my_funcs/ug_funcs.h"

int main(int argc, char *argv[])
{
  SOUND_DATA *in_data=NULL,*out_data=NULL;
  mus_sample_t **obuf, **ibuf;
  int i;

  void process(SOUND_DATA *in,SOUND_DATA *out, mus_sample_t **ibuf,mus_sample_t **obuf );
    
  if(checkin(argc, 3, "Command line: snd-randi infile outfile", "SND-RANDI")==-1) {
    getchar();
    return(-1);
  }

  mus_sound_initialize();
  
  if((in_data=open_input(argv[1],  "\nINPUT FILE:"))==NULL)
    {return(-1);}
  if((out_data=open_output(argv[2],"\nOUTPUT FILE:",in_data))==NULL)
    {return(-1);}
  
  /*RESERVAR MEMORIA*/
  ibuf=snd_memalloc(in_data, BUFFER_SIZE);
  obuf=snd_memalloc(out_data, BUFFER_SIZE);
  
  /*PROCESAR*/
  process(in_data, out_data, ibuf, obuf);

  /*LIBERAR MEMORIA, CERRAR ARCHIVOS*/
  snd_dealloc(in_data, BUFFER_SIZE, ibuf);
  snd_dealloc(out_data, BUFFER_SIZE, obuf);
  close(in_data->fd);
  close(out_data->fd);
  free(in_data);
  free(out_data);
  
  return(0);
}

/***************************************************************/
void process(SOUND_DATA *in,SOUND_DATA *out, mus_sample_t **ibuf,mus_sample_t **obuf)
{
  int i,k,j;
  int size=BUFFER_SIZE;
  float fstep, rfrec;
  RANDI *randat;
  float insamp, outsamp;
  float maxamp=0.;
  
  srand(322);

  rfrec =5.;
  fstep =100. / (float)out->frames;
  randat=(RANDI*)set_randi(out->srate, rfrec);

  fprintf(stdout, "\n  Randmax: %f ",RAND_MAX);

  for (i=0; i < out->frames; i+=BUFFER_SIZE) {
      mus_sound_read (in->fd, 0,size-1,in->chans, ibuf);    
      for (k=0; k<in->chans; k++) {

	for (j=0; j<size; j++) {
	  if(i+j == out->frames) break; 
	  insamp=MUS_SAMPLE_TO_FLOAT(ibuf[k][j]);
	  outsamp= insamp * randi(randat, rfrec+fstep , out->srate);
	  rfrec+=fstep;
	  obuf[k][j]=MUS_FLOAT_TO_SAMPLE(outsamp);
	  maxamp=(outsamp > maxamp? outsamp : maxamp);
	}

      }  
      if(j != BUFFER_SIZE && j !=0) size=j-1;
	mus_sound_write(out->fd,0,size-1,out->chans,obuf);
  }
  mus_sound_close_input (in->fd); 
  mus_sound_close_output(out->fd, out->frames * mus_bytes_per_sample(out->format));

  fprintf(stdout,"\n\n MAXAMP=%5.3f", maxamp);
  fprintf(stdout,"\n\n %ld frames of %ld samples each one written", ((i-BUFFER_SIZE)+j),out->chans );

  free(randat);
  return;
}
/***************************************************************/
